#include "mazeclass.as"


// the mazeclass is general-purpose engouh to be used to create mazes of 
// all types, as long as they can be divided into discrete cells.  
//
// in this example, the maze has cells with curved edges, and the cell
// addresses are converted to a form of polar coordinates, rather than
// grid coordinates

// the entrance and exit of the maze  is unimportant, since the entire maze 
// is interconnected - each position
// can be reached from any other position using one unique path (the
// construction method guarantees that there are no loops or circuits).
//
// In this maze, the exit (or destination) is in the center of the maze
//
//

degToRad = Math.PI/180;

function drawCWArc(mc, x,y,radius,bA,eA)	// this is too crude for circles, but okay for roundrects
{
	// trace("rad = " + radius);
	if (eA < bA)
		eA += 360;
	var r = radius;
	var theta = 45*degToRad;
	var cr = radius/Math.cos(theta/2);
	var angle = bA*degToRad;
	var cangle = angle-theta/2;
	var n = Math.floor((eA-bA)/45);
	mc.moveTo(x+r*Math.cos(angle), y+r*Math.sin(angle));
	for (var i=0;i < n;i++) 
	{
		angle += theta;
		cangle += theta;
		var endX = r*Math.cos (angle);
		var endY = r*Math.sin (angle);
		var cX = cr*Math.cos (cangle);
		var cY = cr*Math.sin (cangle);
		mc.curveTo(x+cX,y+cY, x+endX,y+endY);
	}
	// final bit
	theta = (eA-(bA+45*n))*degToRad;
	if (theta > 0.01) {	// arbitrary limit
		cr = radius/Math.cos(theta/2);
		angle += theta;
		cangle = angle-theta/2;
		var endX = r*Math.cos (angle);
		var endY = r*Math.sin (angle);
		var cX = cr*Math.cos (cangle);
		var cY = cr*Math.sin (cangle);
		mc.curveTo(x+cX,y+cY, x+endX,y+endY);
	}
}

function myDrawLine(mc, x1, y1, x2, y2)
{
		mc.moveTo(x1, y1);
		mc.lineTo(x2, y2);
}

function myDrawArc(mc, r, sa, ex)
{
	if (ex < 20) {	// some arbitrary limit at which point straight is fine
		mc.moveTo(Math.cos(sa*degToRad)*r, Math.sin(sa*degToRad)*r);
		mc.lineTo(Math.cos((sa+ex)*degToRad)*r, Math.sin((sa+ex)*degToRad)*r);
	}
	else
		drawCWArc(mc, 0, 0, r, sa, sa+ex);
}

function PolarMaze(innerC, splits, ringWidth)
{
	this.innerC = innerC;
	this.splits = splits;
	this.ringWidth = ringWidth;
	this.nbrRings = splits.length;
	this.startRingAddr = new Array(this.nbrRings);
	this.cellsPerRing = new Array(this.nbrRings);
	var addr = 0;
	var ringCnt = innerC;
	for (var i = 0; i < this.nbrRings; ++i)
	{
		this.startRingAddr[i] = addr;
		ringCnt *= splits[i];
		this.cellsPerRing[i] = ringCnt;
		addr += ringCnt;
	}
	this.nbrCells = addr;
	this.digMethod = 2;	// use 1 for "snakier"

	this.idleReport = spinDice;
	this.completionFunc = FinishMaze;
	
}

PolarMaze.prototype = new Maze();

PolarMaze.prototype.pokeEntryHoles = function()
{
	this.cells[Math.floor(Math.random()*this.innerC)].clearEdge(0);
	var lastCell = this.startRingAddr[this.nbrRings-1] + Math.floor(Math.random()*this.cellsPerRing[this.nbrRings-1]);
	this.cells[lastCell].clearEdge(3);
}


PolarMaze.prototype.getNbrEdges = function(addr)
{
    var sring = this.getRingFromAddr(addr);
    var scnt = addr-this.startRingAddr[sring];
    if (sring >= this.nbrRings - 1)
      return 4;
    else
      return 3 + this.splits[sring+1];
}

PolarMaze.prototype.getMaxCells = function()
{
	return this.nbrCells;
}

// Given a cell, determine which ring it is located in
PolarMaze.prototype.getRingFromAddr = function(addr)
{
   for (var i = 1; i < this.nbrRings; ++i)
    {
      if (addr < this.startRingAddr[i])
        return i-1;
    }
	  return this.nbrRings-1;
}

PolarMaze.prototype.findNeighbors = function(cellAddr)
{
	// trace("Getting neighbors for " + cellAddr);
	var vec = new CellList(this.getNbrEdges(cellAddr));


	var sring = this.getRingFromAddr(cellAddr);
	var scnt = cellAddr - this.startRingAddr[sring];

	if (sring > 0)
		vec.cells[0] = this.cells[ this.startRingAddr[sring-1] + Math.floor(scnt / this.splits[sring]) ];

	vec.cells[1] = this.cells[ this.startRingAddr[sring] + (scnt == this.cellsPerRing[sring]-1? 0 : scnt+1)];

	vec.cells[2] = this.cells[ this.startRingAddr[sring] + (scnt == 0? this.cellsPerRing[sring]-1 : scnt-1)];

	if (sring < this.nbrRings-1)
	{
		var sf = this.splits[sring+1];
		for (var i = 0; i < sf; ++i)
			vec.cells[3+i] = this.cells[ this.startRingAddr[sring+1] + scnt * sf + i];
	}

	return vec;
}



PolarMaze.prototype.drawCell = function(mc, cell)
{
	var sring = this.getRingFromAddr(cell.address);
	var scnt = cell.address-this.startRingAddr[sring];
	var sa = scnt*360.0/this.cellsPerRing[sring];
	var ex = 360.0/this.cellsPerRing[sring];

	// spiral color effect
	var t = this.t;
	var chase = sa*(6+Math.sin(t * 0.5)*5) + 
	            scnt * Math.sin(t * 0.1)*180 + 
	            t * 360;
	chase = chase * degToRad;
	var red  = 127+Math.sin(chase)*127;
	var grn  = 127+Math.sin(chase+2+Math.sin(t)*2)*127;
	var blu = 127+Math.sin(chase+4+Math.sin(t)*2)*127;
	var clr = (red << 16) | (grn << 8) | blu;
	// trace("Drawing " + x + " " + y);

	// Interior
/*
	{
			var r = this.ringWidth + sring * this.ringWidth;
			var r2 = this.ringWidth + (sring+1) * this.ringWidth;
			var  a1 = (sa) * degToRad;
			var  a2 = (sa+ex) * degToRad;
		mc.moveTo(Math.cos(a1)*r, Math.sin(a1)*r);
		mc.lineStyle(2, clr, 100);
	 	mc.beginFill(clr,100);
		mc.lineTo(Math.cos(a2)*r,  Math.sin(a2)*r);
		mc.lineTo(Math.cos(a2)*r2, Math.sin(a2)*r2);
		mc.lineTo(Math.cos(a1)*r2, Math.sin(a1)*r2);
		mc.lineTo(Math.cos(a1)*r,  Math.sin(a1)*r);
		mc.endFill();
	}
*/
	mc.lineStyle(2, clr, 100);

	if (cell.edgeFlags & 1) {
			var r = this.ringWidth+sring*this.ringWidth;
			myDrawArc(mc, r, sa, ex);
	}

	if (cell.edgeFlags & 2) {
			var r = this.ringWidth + sring * this.ringWidth;
			var r2 = this.ringWidth + (sring+1) * this.ringWidth;
			var  angle = (sa+ex) * degToRad;
			myDrawLine(mc, (Math.cos(angle)*r),
								     (Math.sin(angle)*r),
								     (Math.cos(angle)*r2),
								     (Math.sin(angle)*r2));
	}

	// Only need to draw this edge on first cell of each ring
	if (scnt == 0 && (cell.edgeFlags & 4))
	{
			var r = this.ringWidth + sring * this.ringWidth;
			var r2 = this.ringWidth + (sring+1) * this.ringWidth;
			var  angle = sa * degToRad;
			myDrawLine(mc, (Math.cos(angle)*r),
								     (Math.sin(angle)*r),
								     (Math.cos(angle)*r2),
								     (Math.sin(angle)*r2));
	}


	if (sring == this.nbrRings-1) {
		// mc.lineStyle(2, Math.floor(Math.random()*0xFFFFFF), cTint);
		var sf = (sring == this.nbrRings-1? 1 : this.splits[sring+1]);
		for (var i = 0; i < sf; ++i)
		{	// draw edge 3+
			if ((cell.edgeFlags & (1 << (3+i))) > 0)

			{
				var r = this.ringWidth + (sring+1) * this.ringWidth;
				// trace("drawarc " + r + " " + sa+i*ex/sf + " " + ex/sf);
				myDrawArc(mc, r, sa+i*ex/sf, ex/sf);
			}
		}
	}
}

function drawMyMaze()
{
	// trace("Draw My Maze");
	// this._rotation += 1;
	mc.clear();
	mc.lineStyle(2, 0x000000, 100);
	this.maze.t = getTimer()*.001;
	for (var i = this.maze.cells.length; --i >= 0;)
	{
		this.maze.drawCell(this, this.maze.cells[i]);
	}
	this.onEnterFrame = undefined;
}

function spinDice(r)
{
	dice_mc._rotation = r*360;	
}


function animButtonGrow()
{
	delta = this.tScale - this._xscale;
	if (delta >= 1) 
	{
		this._xscale += delta/2;
		this._yscale += delta/2;
	}
	else {
		this._xscale = this.tScale;
		this._yscale = this.tScale;
		this.onEnterFrame = undefined;
	}
}

function growButton()
{
	this.tScale = 125;
	this.active = true;
	this.onEnterFrame = animButtonGrow;
}

function shrinkButton()
{
	this.tScale = 100;
	this.active = false;
	this.onEnterFrame = animButtonGrow;
}

function FinishMaze()
{
	mc.onEnterFrame = drawMyMaze;
}

function resetPolarMaze()
{
	if (this.active)
		mc.maze.init();
}


function init()
{

// var splits = [1,2,1,2,1,1,1,2,1,1,1,1,1,1,2,1,1];
	var splits =  [1,2,1,2,1,1,1,2,1,1];

	this.mc = this.createEmptyMovieClip("maze_mc", 0);
	this.mc._x = 150;	// movie is shifted to put center of maze at center
	this.mc._y = 150;
	this.mc.maze = new PolarMaze(8, splits, 300/(splits.length*2+3));
	this.mc.maze.init();

	this.dice_mc.onMouseUp = resetPolarMaze;
	this.dice_mc.onRollOver = growButton;
	this.dice_mc.onRollOut =  shrinkButton;

}

init();
