// mazeclass.as - Jim Bumgardner
//
// A set of classes for making cell-based mazes - these mazes can
// take any shape, as long as each cell has a finite/known number
// of edges, which can be determined by the cell's address from 0-n.
// Each edge must connect to either another cell, or the outside.
//
//
++initCtr;
if (initCtr > 1)
	trace("Mazeclass script called twice");


// An individual maze cell
function MazeCell(address, nbrEdges) // MazeCell constructor
{
	this.address = address;
	this.edgeFlags = 0;
	this.visited = false;
	for (var i = 0; i < nbrEdges; ++i)
	{
		this.edgeFlags <<= 1;	// set sealed edge for each edge
		this.edgeFlags |= 1;
	}
}

MazeCell.prototype.clearEdge = function(edgeNbr)
{
	this.edgeFlags &= ~(1 << edgeNbr);
}

function CellList(nbrCells) // CellList constructor
{
	this.cells = new Array();
	for (var i = 0; i < nbrCells; ++i)
		cells[i] = null;
}

CellList.prototype.getRandomEdge = function()
{
    var nsize = this.countNonEmpties();
    var nIdx = random(nsize);
    var n = 0;
    for (var i = this.cells.length; --i >= 0;)
    {
      if (this.cells[i] != null && !(this.cells[i].visited) && n++ == nIdx)
        return i;
    }
    return -1;
}

CellList.prototype.containsNonEmpties = function()
{
  for (var i = this.cells.length; --i >= 0;)
	{
		if (this.cells[i] != null)
			return true;
	}
	return false;
}

CellList.prototype.countNonEmpties = function()
{
	var sum = 0;
  for (var i = this.cells.length; --i >= 0;)
	{
		if (this.cells[i] != null)
			sum++;
	}
	return sum;
}

function Maze()		// Maze Constructor
{
	this.digMethod = 0;
}

Maze.prototype.clearVisited = function()
{
	for (var i = 0; i < this.maxCells; ++i)
	{
		this.cells[i].visited = false;
	}
	this.cstack = new Array();
}

Maze.prototype.solvePass = function()
{
	for (var i = 0; i < 25 && this.nbrSolved < this.maxCells && this.n++ < this.maxCells*2; ++i)
	{
		var vec = this.findOpenNeighbors(this.curCellAddr);

		var	cell = this.cells[this.curCellAddr];
		cell.visited = true;

		if (vec.containsNonEmpties())
		{
			var edgeNbr = vec.getRandomEdge();
			var newcell = vec.cells[edgeNbr];
			cell.clearEdge(edgeNbr);
			newcell.clearEdge(this.getEdge(newcell, cell));
			// newcell.visited = true;
			++this.nbrSolved;
			this.cstack.push(cell.address);
			// trace("push " + cell.address);
			this.curCellAddr = newcell.address;
		}
		else {
			if (this.cstack.length <= 0)
			{
				trace("Empty Stack (shouldn't happen)!");
				clearInterval(this.sih);
				if (this.completionFunc != undefined)
					this.completionFunc();
				return;
			}
			switch (this.digMethod) {
			case 0:
				// METHOD A (FIFO)
				this.curCellAddr = this.cstack.shift();
				break;
			case 1:
				// METHOD B (LIFO recursive back-tracer) "snakier"
				this.curCellAddr = this.cstack.pop();
				break;
			case 2:
				// METHOD A ("hunt and kill" - random) (in the middle)
				var idx = random(this.cstack.length);
				this.curCellAddr = this.cstack[idx];
				this.cstack.splice(idx,1);
				break;
			}
		}
	}
	if (this.nbrSolved >= this.maxCells)
	{
		// trace("maze solved");
		if (this.idleReport != undefined)
			this.idleReport(1);
		clearInterval(this.sih);
		if (this.completionFunc != undefined)
			this.completionFunc();
	}
	else if (this.n >= this.maxCells*2)
	{
		trace("maze not solved");
		clearInterval(this.sih);
		if (this.completionFunc != undefined)
			this.completionFunc();
	}
	else {
		// trace("nbr solved = " + this.nbrSolved);
		if (this.idleReport != undefined)
			this.idleReport(this.nbrSolved/this.maxCells);
	}
}

Maze.prototype.init = function()
{
	this.maxCells = this.getMaxCells();

	// trace("Init maxcells = " + this.maxCells);

	this.cells = new Array();
	this.cstack = new Array();
	for (var i = 0; i < this.maxCells; ++i)
	{
		this.cells[i] = new MazeCell(i, this.getNbrEdges(i));
	}
	this.pokeEntryHoles();

	// Start on random cell, create paths...
	this.curCellAddr = random(this.maxCells);
	this.nbrSolved = 1;
	this.n = 0;
	this.sih = SetInterval( function(maze) { maze.solvePass(); }, 50, this);
}



Maze.prototype.getEdge = function(src,dst)
{
	var vec = this.findNeighbors(src.address);
	for (var i = vec.cells.length; --i >= 0; )
	{
		if (vec.cells[i] == dst)
			return i;
	}
	return -1;
}

Maze.prototype.findOpenNeighbors = function(addr)
{
	var vec = this.findNeighbors(addr);
	for (var i = vec.cells.length; --i >= 0; )
	{
		if (vec.cells[i] != null && vec.cells[i].visited)
			vec.cells[i] = null;
	}
	return vec;
}

// used for traversing a finished maze
Maze.prototype.findOpenAccess = function(addr)
{
	var vec = this.findNeighbors(addr);
	var thiscell = this.cells[addr];
	for (var i = vec.cells.length; --i >= 0; )
	{
		if (vec.cells[i] != null && (vec.cells[i].visited || (thiscell.edgeFlags & (1 << i)) != 0))
			vec.cells[i] = null;
	}
	return vec;
}



// These are basic grid-maze style settings
// and should be overridden by instance
Maze.prototype.pokeEntryHoles = function()
{
	this.cells[0].clearEdge(1);
	this.cells[this.maxCells-1].clearEdge(3);
}

// Maze.prototype.getMaxCells = function()
// Maze.prototype.getNbrEdges = function(addr)
// Maze.prototype.findNeighbors = function(addr)

// Maze.prototype.drawCell = function(mc,cell)   ????
// Maze.prototype.initMC= function(mc)      ????

